/*
    WiiCade Wii Remote API v2.0 
    Copyright (C) 2007 Jerason Banes

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    
    It is understood that no reaonable library system exists for Flash
    programs. To that end, this library is considered to be in library
    form when distributed as a source code package for linking to 
    Flash and Actionscript programs. Similarly, static linking of the
    library is understood to be the preferred form of distributing
    the library in binary form.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

import com.wiicade.*;

class com.wiicade.KeyboardController
{
    private var bitmap:Number = 0;
    private var callback:Function;
    
    private static var keyCodes:Array;
    private static var buttonCodes:Array;
    
    function KeyboardController(callback:Function)
    {
        this.callback = callback;
        
        Key.addListener(this);
        Mouse.addListener(this);

        if(!keyCodes) init();
    }
    
    static function init():Void
    {
        keyCodes = new Array();
        buttonCodes = new Array();

        keyCodes[37] = WiiRemote.BUTTON_LEFT;
        keyCodes[38] = WiiRemote.BUTTON_UP;
        keyCodes[39] = WiiRemote.BUTTON_RIGHT;
        keyCodes[40] = WiiRemote.BUTTON_DOWN;

        keyCodes[65] = WiiRemote.BUTTON_1;
        keyCodes[83] = WiiRemote.BUTTON_2;
        keyCodes[68] = WiiRemote.BUTTON_A;
        keyCodes[32] = WiiRemote.BUTTON_B;

        keyCodes[187] = WiiRemote.BUTTON_PLUS;
        keyCodes[189] = WiiRemote.BUTTON_MINUS;

        buttonCodes[WiiRemote.BUTTON_LEFT] = 37;
        buttonCodes[WiiRemote.BUTTON_UP] = 38;
        buttonCodes[WiiRemote.BUTTON_RIGHT] = 39;
        buttonCodes[WiiRemote.BUTTON_DOWN] = 40;

        buttonCodes[WiiRemote.BUTTON_1] = 65;
        buttonCodes[WiiRemote.BUTTON_2] = 83;
        buttonCodes[WiiRemote.BUTTON_A] = 68;
        buttonCodes[WiiRemote.BUTTON_B] = 32;

        buttonCodes[WiiRemote.BUTTON_PLUS] = 187;
        buttonCodes[WiiRemote.BUTTON_MINUS] = 189;
    }
    
    private function onKeyDown():Void
    {
        var code = Key.getCode();
      
        if(keyCodes[code])
        {
            this.bitmap |= keyCodes[code];
            callback((Wii.COMMAND_BUTTONS << 16) | this.bitmap);
        }
    }
    
    public function onKeyUp():Void
    {
        var code = Key.getCode();

        if(keyCodes[code])
        {
            this.bitmap &= ~keyCodes[code];
            callback((Wii.COMMAND_BUTTONS << 16) | this.bitmap);
        }
    }
    
    private function onMouseDown():Void
    {
        this.bitmap |= WiiRemote.BUTTON_A;
        callback((Wii.COMMAND_BUTTONS << 16) | this.bitmap);
    }
    
    private function onMouseUp():Void
    {
        this.bitmap &= ~WiiRemote.BUTTON_A;
        callback((Wii.COMMAND_BUTTONS << 16) | this.bitmap);
    }
    
    private function onMouseMove():Void
    {
        callback((Wii.COMMAND_CURSORX << 16) | _xmouse);
        callback((Wii.COMMAND_CURSORY << 16) | _ymouse);
    }
    
    static public function getButtonMappingForKey(keyCode:Number):Number
    {
        if(!keyCodes) Wii.isWii();
        
        return keyCodes[keyCode];
    }

    static public function getKeyMappingForButton(buttonCode:Number):Number
    {
        if(!keyCodes) Wii.isWii();
        
        return buttonCodes[buttonCode];
    }

    static public function setKeyMapping(buttonCode:Number, keyCode:Number):Void
    {
        if(!keyCodes) Wii.isWii();
        
        var origKeyCode = buttonCodes[buttonCode];
        var origButtonCode = keyCodes[keyCode];
        
        if(origKeyCode) keyCodes[origKeyCode] = null;
        if(origButtonCode) buttonCodes[origButtonCode] = null;

        keyCodes[keyCode] = buttonCode;
        buttonCodes[buttonCode] = keyCode;
    }
}
